/*
 * vim: ts=4 sw=4 fdm=marker noet tw=78 ai
 * Copyright (c) 2012, Mel Flynn
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/cdefs.h>
#ifdef __FBSDID
__FBSDID("$Coar: trunk/ldapenter/fetch.c 2 2012-01-10 23:05:07Z rflynn@acsalaska.net $");
#endif

#include <sys/types.h>

#include <assert.h>
#include <err.h>

#include <lber_types.h>
#include <ldap.h>

#include "extern.h"
#include "search.h"
#include "fetch.h"

int
lde_fetch_operational(runtime_t *ctx, char *type, lde_found_cb fn)
{
	int 			  retval; // Return value
	int				  entrycnt; // Entry count
	LDAPMessage 	 *resp = NULL; // Search response from server
	LDAPMessage 	 *cur; // Current entry
	struct berval	**vals; // Value array in BER values
	size_t			  i; // Iterator for value array

	assert(ctx != NULL);
	assert(ctx->ld != NULL);
	TRACEIN(ctx, __func__, "ctx->ld=%p", ctx->ld);

	retval = lde_search_operational(ctx, type, &resp);
	if( retval != LDAP_SUCCESS )
		goto out;

	if( (entrycnt = ldap_count_entries(ctx->ld, resp)) > 0 )
	{
		LOG(ctx, 1, "Processing %i entries", entrycnt);
		for(
				cur = ldap_first_entry(ctx->ld, resp);
				cur != NULL;
				cur = ldap_next_entry(ctx->ld, cur)
		   )
		{
			vals = ldap_get_values_len(ctx->ld, cur, type);
			for( i=0; vals[i] != NULL; i++ )
			{
				if( vals[i]->bv_val != NULL )
					fn(ctx, vals[i]->bv_val);
				else
					warnx("Attribute with NULL value skipped.");
			}// value iterator
			ldap_value_free_len(vals);
		}// entry iterator
	}
	else
	{
		warnx("No entries received. Check server ACL");
		retval = LDAP_OPERATIONS_ERROR;
	}

out:
	if( resp )
		ldap_msgfree(resp);
	TRACEOUT(ctx, __func__, "Returning 0x%02x", retval);
	return (retval);

}
