/*
 * vim: ts=4 sw=4 fdm=marker noet tw=78 ai
 * Copyright (c) 2012, Mel Flynn
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/cdefs.h>
#ifdef __FBSDID
__FBSDID("$Coar: trunk/ldapenter/search.c 2 2012-01-10 23:05:07Z rflynn@acsalaska.net $");
#endif

#include <assert.h>
#include <err.h>

#include <ldap.h>

#include "extern.h"
#include "search.h"

int
lde_search_entry(runtime_t *ctx, LDAPMessage **result)
{
	int res;

	TRACEIN(ctx, __func__, "filter=%s", ctx->arg_entry);
	*result = NULL;

	res = ldap_search_ext_s(ctx->ld, ctx->opt_base, ctx->opt_scope,
			ctx->arg_entry, NULL, 0, NULL, NULL, NULL, -1, result);

	TRACEOUT(ctx, __func__, "returning %i", res);
	return (res);
}

int
lde_search_operational(runtime_t *ctx, char *type, LDAPMessage **result)
{
	char *attrs[2] = { type, NULL };
	int res;

	assert(ctx != NULL);
	assert(ctx->ld != NULL);
	assert(type != NULL);
	TRACEIN(ctx, __func__, "sending operational query for %s", type);
	*result = NULL;
	res = ldap_search_ext_s(ctx->ld, "cn=subschema", LDAP_SCOPE_BASE,
			NULL, attrs, 0, NULL, NULL, NULL, -1, result);
	TRACEOUT(ctx, __func__, "Returning 0x%02x and result %p", res, *result);
	return (res);
}
